@@ -1,6 +1,5 @@
 /* Reading Desktop Entry files.
-   Copyright (C) 1995-1998, 2000-2003, 2005-2006, 2008-2009, 2014-2019 Free
-   Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2003, 2005-2006, 2008-2009, 2014-2019, 2023 Free Software Foundation, Inc.
    This file was written by Daiki Ueno <ueno@gnu.org>.
 
    This program is free software: you can redistribute it and/or modify
@@ -43,8 +42,6 @@
 
 #define _(str) gettext (str)
 
-#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
-
 /* The syntax of a Desktop Entry file is defined at
    https://standards.freedesktop.org/desktop-entry-spec/latest/index.html.  */
 
@@ -252,24 +249,20 @@ desktop_lex (token_ty *tp)
             for (;;)
               {
                 c = phase2_getc ();
-                switch (c)
+                if (c == EOF || c == ']')
+                  break;
+                if (c == '\n')
                   {
-                  default:
-                    /* Group names may contain all ASCII characters
-                       except for '[' and ']' and control characters.  */
-                    if (!(c_isascii (c) && c != '[' && !c_iscntrl (c)))
-                      break;
-                    APPEND (c);
-                    continue;
-                  case '\n':
                     po_xerror (PO_SEVERITY_WARNING, NULL,
                                real_file_name, gram_pos.line_number, 0, false,
                                _("unterminated group name"));
                     break;
-                  case EOF: case ']':
-                    break;
                   }
-                break;
+                /* Group names may contain all ASCII characters
+                   except for '[' and ']' and control characters.  */
+                if (!(c_isascii (c) && c != '[' && !c_iscntrl (c)))
+                  break;
+                APPEND (c);
               }
             /* Skip until newline.  */
             while (c != '\n' && c != EOF)
@@ -296,15 +289,9 @@ desktop_lex (token_ty *tp)
             for (;;)
               {
                 c = phase2_getc ();
-                switch (c)
-                  {
-                  default:
-                    APPEND (c);
-                    continue;
-                  case EOF: case '\n':
-                    break;
-                  }
-                break;
+                if (c == EOF || c == '\n')
+                  break;
+                APPEND (c);
               }
             APPEND (0);
             tp->type = token_type_comment;
@@ -360,15 +347,9 @@ desktop_lex (token_ty *tp)
                     for (;;)
                       {
                         int c2 = phase2_getc ();
-                        switch (c2)
-                          {
-                          default:
-                            APPEND (c2);
-                            continue;
-                          case EOF: case ']':
-                            break;
-                          }
-                        break;
+                        if (c2 == EOF || c2 == ']')
+                          break;
+                        APPEND (c2);
                       }
                     break;
 
@@ -386,9 +367,9 @@ desktop_lex (token_ty *tp)
                 c = phase2_getc ();
                 switch (c)
                   {
+                  case ' ':
+                    continue;
                   default:
-                    if (c == ' ')
-                      continue;
                     phase2_ungetc (c);
                     break;
                   case EOF: case '\n':
@@ -419,9 +400,9 @@ desktop_lex (token_ty *tp)
                 c = phase2_getc ();
                 switch (c)
                   {
+                  case ' ':
+                    continue;
                   default:
-                    if (c == ' ')
-                      continue;
                     phase2_ungetc (c);
                     break;
                   case EOF:
